import { assert, assertEquals } from "https://deno.land/std@0.102.0/testing/asserts.ts";
import { Label, Forward, Backward } from "./nn.ts";
import { default as M } from "./m.ts";

const input = [
    [ 0.1,  0.05],
    [ 0.0, -0.06]
    [ 0.99, 0.85],
    [ 1.2,  1.05]
];

const training = [];
const stages = [];
const layers = [];

Deno.test("NN.Label", ()=>
{
    Label(training,
    [
        [ 0.1,  0.05],
        [ 0.0, -0.06]
    ],
    [1]);
    Label(training,
    [
        [ 0.99, 0.85],
        [ 1.2,  1.05]
    ],
    [0]);
    stages.push(training[0]);
    console.log(training);
    assertEquals(training.length, 2, "input and output sets created");
    assertEquals(training[0].length, training[1].length, "both sets have same length");
    assertEquals(training[0][0].length, 3, "padded input component");
    assertEquals(training[1][0].length, 1, "unchanged label vector");
});

Deno.test("NN.Backward", ()=>
{

    layers.push(M.Create.Box([0, 0, 0], [1, 1, 1], 4));
    layers.push(M.Create.Box([0, 0, 0, 0, 0], [1, 1, 1, 1, 1], 1));

    let copy1 = M.Create.Clone(layers[0]);
    let copy2 = M.Create.Clone(layers[1]);

    for(let i=0; i<1000; i++)
    {
        Backward(stages, layers, training[1], 0.1);
    }

    assert(layers[0][0][0] != copy1[0][0][0], "first matrix has changed");
    assert(layers[1][0][0] != copy2[0][0][0], "second matrix has changed");
});

Deno.test("NN.Forward", ()=>
{
    console.log(Forward(stages, layers));
    console.log(training[1]);
});

